#!/bin/busybox sh

. /boot/mycube/updater/scripts/config


USBDEV=/dev/sd??
MYCUBEDBARCHIVE="mycube.updater.tar"
MYCUBEDBPATH=root/d2002/database



#----------------------------------------------------
# detectConfig : 
#	read mycube	configuration file (/boot/mycube/conf/mycube.properties) 
#	in order to extract current version name and date and properties to
#	be persisted such as WiFi settings.
detectConfig() {
    echo "================================================="
    echo "        Detection de la config actuelle         "
    echo "================================================="
	mymount $BOOTPARTITION
	BOOTMOUNT=$RES

	#Detect if it is possible to rollback to previous version
	if [ -f $BOOTMOUNT/$CONFIG_ORIGINAL ];then
		echo "Fichier $BOOTMOUNT/$CONFIG_ORIGINAL trouvé"
		hput data config.txt 1
	else
		hput data config.txt 0
		echo "Fichier $BOOTMOUNT/$CONFIG_ORIGINAL introuvable"
	fi
	
	#Detect if updater folder exist
	if [ -d $BOOTMOUNT/$UPDATERFOLDER ];then
		echo "Répertoire $BOOTMOUNT/$UPDATERFOLDER trouvé"
		hput data updaterfolder 1
	else
		hput data updaterfolder 0
		echo "Répertoire $BOOTMOUNT/$UPDATERFOLDER introuvable"
	fi

	if [ -f $BOOTMOUNT/$MYCUBE_PROPERTIES ];then
		
		hput data mycube.sd.name `get_prop $BOOTMOUNT/$MYCUBE_PROPERTIES mycube.sd.name`
		
		CODE=`get_prop $BOOTMOUNT/$MYCUBE_PROPERTIES mycube.sd.date`
		hput data mycube.sd.code $CODE
		hput data mycube.sd.date `echo $CODE | awk -F"-" '{printf "%02d-%02d-%04d"1$3,$2,$1}'`

		#hput data network.hostapd.channel `get_prop $BOOTMOUNT/$MYCUBE_PROPERTIES network.hostapd.channel`
		#hput data network.hostapd.key `get_prop $BOOTMOUNT/$MYCUBE_PROPERTIES network.hostapd.key`
		#hput data network.notfound `get_prop $BOOTMOUNT/$MYCUBE_PROPERTIES network.notfound`

		echo "Version actuelle de la carte SD : `hget data mycube.sd.name` datée du `hget data mycube.sd.date2` (`hget data mycube.sd.code`)"
		#echo "Canal Wifi : [`hget data network.hostapd.channel`]"
		#echo "Clé Wifi : [`hget data network.hostapd.key`]"
		#echo "NOT FOUND : [`hget data network.notfound`]"
	fi

	myumount $BOOTPARTITION   
}


#----------------------------------------------------
# restoreConfig : 
#	update mycube configuration file (/boot/mycube/conf/mycube.properties) 
#	with properties to be persisted such as WiFi settings.
restoreConfig() {
    echo "================================================="
    echo "           Restauration de la config             "
    echo "================================================="

	mymount $BOOTPARTITION [ fail "Impossible de monter la partition [$BOOTPARTITION]"
	BOOTMOUNT=$RES
	#if [ "`hget data network.hostapd.channel`" != "" ];then
	#	echo "Restauration de network.hostapd.channel: [`hget data network.hostapd.channel`]"
	#	set_prop "$BOOTMOUNT/$MYCUBE_PROPERTIES" network.hostapd.channel `hget data network.hostapd.channel`
	#fi
	#if [ "`hget data network.hostapd.key`" != "" ];then
	#	echo "Restauration de network.hostapd.key : [`hget data network.hostapd.key`]"
	#	set_prop "$BOOTMOUNT/$MYCUBE_PROPERTIES" network.hostapd.key `hget data network.hostapd.key`
	#fi
	#if [ "`hget data network.notfound`" != "" ];then
	#	echo "Restauration de network.notfound : [`hget data network.notfound`]"
	#	set_prop "$BOOTMOUNT/$MYCUBE_PROPERTIES" network.notfound `hget data network.notfound`
	#fi

	myumount $BOOTPARTITION   
}


#----------------------------------------------------
# detectISO : 
#	Detects all images available on USB key
#	iso.count=x
#	iso.x.name=YELLOW SPRING LITE 2016
#	iso.x.date=10-05-2016
#	iso.x.code=2016-05-10
#	iso.x.device=/dev/sda1
#	iso.x.type=iso|rar
#	iso.x.path=/mnt/dev/sda1/2016-05-10-YELLOW SPRING LITE 2016.rar
detectISO(){
    echo "================================================="
    echo "     Détéction des fichier image de carte SD     "
    echo "================================================="

	#Activate USB support
	echo "Activation des modules USB"
	modprobe sd_mod &> /dev/null || fail "Aucune clé USB trouvée"
	modprobe usb-storage &> /dev/null || fail "modprobe usb-storage failed"
	for i in `seq 1 4`; do
		sleep 1
		echo ".#"
	done
	echo ""

	#Detect ISO file (or RAR file containing ISO file) stored on connected USB devices
	echo "Detection des versions MyCube présentes sur les clés USB :"
	DEVCOUNT=0
	FILECOUNT=0
	for USBPART in $(ls $USBDEV);do
		echo "Analyse de la partition $USBPART"
		DEVCOUNT=$(($DEVCOUNT + 1))
		mymount "$USBPART"
		USBMOUNT=$RES
		
		#Search for ISO file
		for IMGFILE in $(ls $USBMOUNT/*.img);do
			echo "Fichier ISO trouvé en $IMGFILE : #"
			FILENAME=`echo $IMGFILE | awk -F"/" '{print $NF}' | awk -F"." '{print $1}'`
			VERSIONNAME=`echo $FILENAME | awk -F"-" '{print $4}'`
			VERSIONCODE=`echo $FILENAME | awk -F"-" '{printf "%04d-%02d-%02d",$1,$2,$3}'`
			VERSIONDATE=`echo $FILENAME | awk -F"-" '{printf "%02d-%02d-%04d",$2,$2,$1}'`
			if [ "$VERSIONNAME" == "" ];then
				echo "Fichier non valide [$VERSIONNAME]"
			else 
				FILECOUNT=$(($FILECOUNT + 1))
				hput data iso.$FILECOUNT.device $USBPART
				hput data iso.$FILECOUNT.path $IMGFILE
				hput data iso.$FILECOUNT.type iso
				hput data iso.$FILECOUNT.name $VERSIONNAME
				hput data iso.$FILECOUNT.date $VERSIONDATE
				hput data iso.$FILECOUNT.code $VERSIONCODE
				echo "`hget data iso.$FILECOUNT.name` [`hget data iso.$FILECOUNT.type`] daté du `hget data iso.$FILECOUNT.date` sur `hget data iso.$FILECOUNT.device`"
			fi
		done

		#Search for ISO file inside RAR file
		for IMGFILE in $(ls $USBMOUNT/*.rar);do
			echo "Fichier ISO trouvé en $IMGFILE : #"
			FILENAME=`echo $IMGFILE | awk -F"/" '{print $NF}' | awk -F"." '{print $1}'`
			VERSIONNAME=`echo $FILENAME | awk -F"-" '{print $4}'`
			VERSIONCODE=`echo $FILENAME | awk -F"-" '{printf "%04d-%02d-%02d",$1,$2,$3}'`
			VERSIONDATE=`echo $FILENAME | awk -F"-" '{printf "%02d-%02d-%04d",$3,$2,$1}'`
			if [ "$VERSIONNAME" == "" ];then
				echo "Fichier non valide [$VERSIONNAME]"
			else 
				RARCHECK=`unrar l "$IMGFILE" | grep "$FILENAME.img" | wc -l`
				if [ $RARCHECK -eq 1 ];then
					echo "Analyse de l'archive [OK]"
					FILECOUNT=$(($FILECOUNT + 1))
					hput data iso.$FILECOUNT.device $USBPART
					hput data iso.$FILECOUNT.path $IMGFILE
					hput data iso.$FILECOUNT.type rar
					hput data iso.$FILECOUNT.name $VERSIONNAME
					hput data iso.$FILECOUNT.date $VERSIONDATE
					hput data iso.$FILECOUNT.code $VERSIONCODE
					echo "`hget data iso.$FILECOUNT.name` [`hget data iso.$FILECOUNT.type`] daté du `hget data iso.$FILECOUNT.date` sur `hget data iso.$FILECOUNT.device`"
				else 
					echo "Archive non valide [$RARCHECK]"
				fi
			fi
		done

		myumount "$USBPART"
	done
	hput data iso.count $FILECOUNT
	hput data dev.count $DEVCOUNT

}

writeISO(){
    echo "================================================="
    echo "            Ecriture de la carte SD              "
    echo "================================================="
    echo "Nom    : [$VNAME]"
    echo "Date   : [$VDATE]"
    echo "Code   : [$VCODE]"
    echo "Device : [$VDEV]"
    echo "Type   : [$VTYPE]"
    echo "Path   : [$VPATH]"
    
	mymount "$VDEV"

	alert "Mise à jour <I>MyCube</I>"  "L\'écriture de la carte SD va commencer.<p><font color=#f00>Cette opération peut durée jusqu\'à 15 minutes."
	if [ "$VTYPE" == "rar" ]; then
		echo "Extraction de l'image et ecriture de la carte SD"
    	(unrar p -ierr "$VPATH" "$VCODE-$VNAME.img" | dd bs=4M of=$SDDEV) 3>&1 1>&2 2>&3 3>&- | tr -s '\b' '\n'
    	#unrar p -ierr "$VPATH" "$VCODE-$VNAME.img" | dd bs=4M of=$SDDEV
    else
		echo "Lecture de l'image et ecriture de la carte SD"
    	dd bs=4M if="$VPATH" of=$SDDEV
    fi
    echo ""
    
	myumount $VDEV
	echo "Lecture de la table de partition [$SDDEV]"
    blockdev --rereadpt $SDDEV
    ls -l ${SDDEV}*

	alert "Mise à jour <I>MyCube</I>"  "Ecriture de la carte SD terminée."
    
	#time -v dd bs=4M if="/mnt/dev/sda1/2015-11-21-YELLOW AUTUMN 2015.img" of=/dev/mmcblk0
	#        dd bs=4M if="/mnt/dev/sda1/2015-11-21-YELLOW AUTUMN 2015.img" of=/dev/mmcblk0
	#time unrar p -ierr "/mnt/dev/sda1/2016-05-10-YELLOW SPRING LITE 2016.rar" "2016-05-10-YELLOW SPRING LITE 2016.img" | dd bs=4M of=/dev/mmcblk0
	#     unrar p -ierr  /mnt/dev/sda1/2016-05-10-YELLOW SPRING LITE 2016.rar   2016-05-10-YELLOW SPRING LITE 2016.img  | dd bs=4M of=/dev/mmcblk0
	#sudo cp /boot/mycube/updater/config.txt /boot/config.txt
	
	#( unrar p -ierr "/mnt/dev/sda1/2016-05-10-YELLOW SPRING LITE 2016.rar" "2016-05-10-YELLOW SPRING LITE 2016.img" | dd bs=4M of=/dev/null 3>&1 1>&2- 2>&3- ) | sed 's/foo/.../g'
}

backupDB(){
    echo "================================================="
    echo "          Sauvegarde de la base de donnée        "
    echo "================================================="

	mymount $VDEV
	USBMOUNT=$RES
	mymount $ROOTPARTITION
	ROOTMOUNT=$RES

	DBPATH=$ROOTMOUNT/$MYCUBEDBPATH
	TARPATH=$USBMOUNT/$MYCUBEDBARCHIVE
	echo "Chemin d'accès à la base de donnée [$DBPATH]"
	

	if [ -f $TARPATH ];then
		confirm "Mise à jour <I>MyCube</I>"  "Une sauvegarde de base de donnée existe déjà sur cette clé USB" "<button onclick=\'mycube.closemodal(\"keep\")\'>Conserver</button><button onclick=\'mycube.closemodal(\"delete\")\'>Supprimer</button>"
		if [ "$RES" == "delete" ];then
			echo "Suppression de l'archive de base de donnée existante [$TARPATH]"
			rm -f "$TARPATH"
		fi
	fi
	
	if [ ! -f $TARPATH ];then
		if [ ! -d $DBPATH ];then
			alert "Mise à jour <I>MyCube</I>"  "<font color=#f00>Aucune base de donnée trouvée.<br>Une base de démonstration sera générée"
		else
			echo "Creation de l'archive [$TARPATH] à partir du répertoire [$DBPATH]"
			cd "$DBPATH"
			tar -cf "$TARPATH" *
			cd /
		fi
	fi

	myumount $VDEV
	myumount $ROOTPARTITION

}


restoreDB(){
    echo "================================================="
    echo "         Restauration de la base de donnée       "
    echo "================================================="

	mymount $VDEV
	USBMOUNT=$RES
	mymount $ROOTPARTITION
	ROOTMOUNT=$RES

	DBPATH=$ROOTMOUNT/$MYCUBEDBPATH
	TARPATH=$USBMOUNT/$MYCUBEDBARCHIVE

	if [ ! -f "$TARPATH" ];then
		alert "Mise à jour <I>MyCube</I>"  "Génération de la base de donnée de démonstration"
	else
		echo "Liste des fichiers dans la base ORIGINALE"
		ls -l $DBPATH

		#Delete previous version
		echo "Suppression de la base existante [$DBPATH]"
		rm -R $DBPATH
		mkdir -p $DBPATH

		echo "Extraction de l'archive [$TARPATH] dans le répertoire [$DBPATH]"
		cd "$DBPATH"
		tar -xf $TARPATH
		cd /
		if [ -f $DBPATH/MYTRAIN.txt ];then
			echo "Suppression du fichier descripteur de l'archive"
			rm $DBPATH/MYTRAIN.txt
		fi
		
		echo "Liste des fichiers dans la base FINALE"
		ls -l $DBPATH

		echo "Suppression de l'archive de base de donnée [$TARPATH]"
		rm -f "$TARPATH"
	fi

	myumount $VDEV
	myumount $ROOTPARTITION

}

updateMYCUBE() {
	

	
	#DETECT CONFIGURATION
	hinit data
	detectConfig
	if [ `hget data config.txt` -eq 0 ];then
		alert "Mise à jour <I>MyCube</I>" "Carte SD vierge<p><font color=#f00>Mise à jour obligatoire</font>"
	fi



	#DETECT ISO FILES
	detectISO
	if [ `hget data dev.count` -eq 0 ];then
		alert "Mise à jour <I>MyCube</I>" "<font color=#f00>Aucune clé USB détectée</font><p>Inserez une clé USB puis recommencez la procedure de mise à jour"
		echo "Mise à jour annulé : Aucune clé USB détectée"
		exit
	fi	
	if [ `hget data iso.count` -eq 0 ];then
		alert "Mise à jour <I>MyCube</I>" "<font color=#f00>Aucun fichier ISO image trouvé sur les clés USB</font><p>Téléchargez puis copiez sur une clé USB un fichier image de carte SD (téléchargeable sur <a target=\"_blank\" href=\"http://mytrain.fr/cms/index.php/download/itemlist/category/36-mycube-cartes-sd\">http://mytrain.fr</a>)<br>Inserez clé USB puis recommencez la procedure de mise à jour"
		echo "Mise à jour annulé : Aucun fichier ISO image trouvé sur les clés USB"
		exit
	fi

	#ASK USER FOR VERSION
	VBUTTON=""
	INDEX=0
	#for  (( f=1; f<=`hget data iso.count`; f++ ));	do
	while [ "$INDEX" -lt  "`hget data iso.count`" ];do
		INDEX=$(($INDEX + 1))
		SVERSION="<B>`hget data iso.$INDEX.name`</B> du <B>`hget data iso.$INDEX.date` [`hget data iso.$INDEX.type`]</B>"
		if [ "`hget data iso.$INDEX.code`" == "`hget data mycube.sd.code`" ];then
			VBUTTON="$VBUTTON<button width=\'100%\' onclick=\'mycube.closemodal(\"$INDEX\")\'>Re-installer $SVERSION</button><p>"
		else
			VBUTTON="$VBUTTON<button width=\'100%\' onclick=\'mycube.closemodal(\"$INDEX\")\'>Installer $SVERSION</button><p>"
		fi
	done

	#ask user for version to install (+ rollback if it exist)
	confirm "Mise à jour <I>MyCube</I>" "<B>Quelle version désirez vous installer</B><P>$VBUTTON" "<button onclick=\'mycube.closemodal(\"cancel\")\'>Annuler</button>"
	echo "confirm=[$RES]"
		
	if [ "$RES" == "cancel" ];then
		exit
	fi

	VNAME=`hget data iso.$RES.name`
	VDATE=`hget data iso.$RES.date`
	VCODE=`hget data iso.$RES.code`
	VDEV=`hget data iso.$RES.device`
	VTYPE=`hget data iso.$RES.type`
	VPATH=`hget data iso.$RES.path`
	
	if [ "$VPATH" == "" ];then
		alert "Mise à jour <I>MyCube</I>" "Erreur de fonctionnement. MyCube va redémarrer"
		echo "Mise à jour annulée sur erreur"
		echo ""
		myreboot
		exit
	fi

	backupDB
	writeISO
	restoreDB
	restoreConfig	
	
	detectConfig
	if [ `hget data updaterfolder` -eq 0 ];then
		echo "================================================="
		echo "         Restauration fichiers UPDATER           "
		echo "================================================="
		. /boot/mycube/updater/scripts/mysaveboot
	fi
	
	confirm "Mise à jour <I>MyCube</I>" "<B><I>MyCube</I></B> version <B>`hget data mycube.sd.name`</B> du <B>`hget data mycube.sd.date`</B> installé" "<button onclick=\'mycube.closemodal(\"ok\")\'>Redémarrer <B><i>MyCube</i><B></button>"
	
	unset IFS
	echo ""
	echo ""
	echo ""
	echo "Mise à jour terminiée."
	echo ""
	myrebootnormal
}


echo "*************************************************************"
echo "*                                                            "
echo "*   U    U  PPPPP   DDDDD    AAAA   TTTTT  EEEEEE  RRRRR     "
echo "*   U    U  P    P  D    D  A    A    T    E       R    R    "
echo "*   U    U  PPPPP   D    D  AAAAAA    T    EEE     RRRRR     "
echo "*   U    U  P       D    D  A    A    T    E       R   R     "
echo "*    UUUU   P       DDDDD   A    A    T    EEEEEE  R    R    "
echo "*                                                            "
echo "* BUILD_VERSION          : $BUILD_VERSION"
echo "* RCS SCRIPT VERSION     : $RCS_VERSION"
echo "* UPDATER SCRIPT VERSION : $SCRIPT_VERSION"
echo "* UPDATER VERSION        : $UPDATER_VERSION"
echo "*                                                            "
echo "*                                                            "

updateMYCUBE
exit





